//////////////////////////////////////////////////
///////////////Normalmappingshader////////////////
///////////////- Sun and 2 Lights/////////////////
//////////////////////////////////////////////////
////03.05.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define OS_O_Normalmapping


//Normalmappingshader for PS 2.0 supporting 3 lights and the sun
MATERIAL* OS_O_Normalmapping_ps20_mat =
{
	effect =
	"
		//Variables
		float4x4 matWorldViewProj;	
		float4x4 matWorld;
		float3x3 matTangent;
		
		float4 vecSunDir;
		float4 vecSunColor;
		float4 vecLightPos[8];
		float4 vecLightColor[8];
		
		float4 vecViewPos;
		
		float4 vecFog;
		float4 vecLight;
		
		float4 vecSkill41;
		
		
		//Textures
		texture entSkin1; //this is the color map
		texture entSkin2; //this is the normal map
		
		
		//Sampler
		sampler colorMap = sampler_state
		{
		   Texture = <entSkin1>;
		   MinFilter = Linear;
		   MagFilter = Linear;
		   MipFilter = Linear;   
		   AddressU  = wrap;
		   AddressV  = wrap;
		};
		
		sampler bumpMap = sampler_state
		{
		   Texture = <entSkin2>;
		   MinFilter = Linear;
		   MagFilter = Linear;
		   MipFilter = Linear;   
		   AddressU  = wrap;
		   AddressV  = wrap;
		};
		
		
		//Structs
		struct VS_TO_PS
		{
			float4 Pos		: POSITION;
			float Fog		: FOG;
			
			float2 Tex		: TEXCOORD0;
			
			float4 View		: TEXCOORD1;
			float3 Sun		: TEXCOORD2;
		};
		
		
		//Vertexshader
		VS_TO_PS VS(float4 inPos		: POSITION,
						float2 inTex		: TEXCOORD0,
						float3 inNormal	: NORMAL,
						float4 inTangent	: TEXCOORD2	)
		{
			VS_TO_PS Out = (VS_TO_PS)0;
			
			//Calculate the final vertexposition
			Out.Pos = mul(inPos, matWorldViewProj);
			
			//Get the vertexposition in worldspace
			float3 PosWorld = mul(inPos, matWorld);
			
			//Create matTangent
			matTangent[0] = mul(inTangent.xyz, matWorld);
			matTangent[1] = mul(cross(inTangent.xyz, inNormal)*inTangent.w, matWorld);
			matTangent[2] = mul(inNormal, matWorld);
			
			//Pass the TexCoords
			Out.Tex = inTex;
			
			//Viewdir to tangent space
			Out.View.xyz = mul(matTangent, vecViewPos.xyz-PosWorld.xyz);
			Out.View.w = Out.Pos.z;
			
			//Sundir to tangent space
			Out.Sun = normalize(mul(matTangent, -vecSunDir));
			
			//Fog
			Out.Fog = 1 - (distance(PosWorld,vecViewPos)-vecFog.x) * vecFog.z;
			
			return Out;
		}
		
		
		//Pixelshader
		float4 PS(VS_TO_PS In) : COLOR
		{
			float4 Color = tex2D(colorMap,In.Tex);
			clip(Color.a-0.7);
			float4 Bump = tex2D(bumpMap,In.Tex);
			Bump.rgb = Bump.rgb * 2.0f - 1.0f;
			
			Bump.a *= vecSkill41.x;
			
			In.View.xyz = normalize(In.View.xyz);
			
			//Sun
			float3 SunDiff = saturate(dot(Bump.rgb,In.Sun));
			float3 SunReflect = normalize(2 * SunDiff * Bump.rgb - In.Sun);
			float3 SunSpec = pow(saturate(dot(SunReflect,In.View.xyz)), vecSkill41.y)*Bump.a;
			float3 Sun = (Color.rgb*SunDiff+SunSpec) * vecSunColor;
			
			float LightShadow = saturate(4*SunDiff);
			
			//Compute the final color
			Color.rgb = Color.rgb*vecLight.rgb+Sun*LightShadow;
			Color.a = 1.0;
			
			return Color;
		}
		
		
		//Technique
		technique NormalMapping
		{
			pass P0
			{
				alphablendenable = false;
				alphatestenable = false;
				zWriteEnable = true;
				cullmode = none;
				
				// compile shaders
				VertexShader = compile vs_1_1 VS();
				PixelShader  = compile ps_2_0 PS();
			}
		}
	";
}


//assigns the shader to the entity
void OS_O_Normalmapping_init(ENTITY* Ent, var SpecStrength, var SpecIntensity)
{
	if(d3d_shaderversion >= 2020)
	{
		Ent.material = OS_O_Normalmapping_ps20_mat;
		
		Ent.skill41 = floatv(SpecStrength);
		Ent.skill42 = floatv(SpecIntensity);
	}
}

action fx_specBump1()
{
	OS_O_Normalmapping_init(my,1,16);
}