//////////////////////////////////////////////////
///////////Shader for 2 Cubeprojections///////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
/////24.12.2008///////////////by Nils Daumann/////
///////////////www.ssmasters.de///////////////////
//////////////////////////////////////////////////

// Constants given by the engine
float4x4 matWorldViewProj;
float4x4 matWorld;

float4 vecLightPos[8];
float4 vecSkill1;
float4 vecSkill5;
float4 vecSkill9;
float4 vecSkill13;
float4 vecSkill17;

float4 vecLight;
float Ambient = 0.0;


// Textures
Texture mtlSkin1;
Texture mtlSkin2;
Texture mtlSkin3;
Texture mtlSkin4;

samplerCUBE Depth1Sampler = sampler_state
{
	texture = <mtlSkin1>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = wrap;
	AddressV = wrap;
};

samplerCUBE Depth2Sampler = sampler_state
{
	texture = <mtlSkin2>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = wrap;
	AddressV = wrap;
};

texture entSkin1;
sampler ColorMapSampler = sampler_state		// Color map sampler.
{
	Texture = <entSkin1>;
	MipFilter = LINEAR;
};

// Structs
struct SSceneVertexToPixel
{
	float4 Position	: POSITION;
	float3 TexCoords	: TEXCOORD0;
	float4 WorldPos	: TEXCOORD1;
	float4 LightDir1	: TEXCOORD2;
	float4 LightDir2	: TEXCOORD3;
};

// Vertex Shader
SSceneVertexToPixel ShadowedSceneVertexShader( float4 inPos : POSITION, float2 inTexCoords : TEXCOORD0, float3 inNormal : NORMAL)
{
	SSceneVertexToPixel Output = (SSceneVertexToPixel)0;
	
	Output.Position = mul(inPos, matWorldViewProj);
	Output.TexCoords.xy = inTexCoords;
	
	//Fading
	Output.WorldPos  = mul(inPos, matWorld);
	
	//cube-projection
	Output.LightDir1.xyz = Output.WorldPos.xyz - vecSkill1.xyz;
	Output.LightDir2.xyz = Output.WorldPos.xyz - vecSkill5.xyz;
	
	return Output;
}

float4 ShadowedScenePixelShader(SSceneVertexToPixel PSIn) : COLOR
{
	clip(tex2D(ColorMapSampler,PSIn.TexCoords.xy).a-0.5);
	
	float3x3 rot;
	
	//rotate dir 1
	rot[0] = float3(	1.0,	0.0,					0.0					);
	rot[1] = float3(	0.0,	cos(vecSkill9.z),	-sin(vecSkill9.z)	);
	rot[2] = float3(	0.0,	sin(vecSkill9.z),	cos(vecSkill9.z)	);
	
	PSIn.LightDir1.xyz = mul(normalize(PSIn.LightDir1.xyz),rot);
	
	rot[0] = float3(	cos(vecSkill9.x) ,	0.0,	sin(vecSkill9.x)	);
	rot[1] = float3(	0.0,						1.0,	0.0					);
	rot[2] = float3(	-sin(vecSkill9.x),	0.0,	cos(vecSkill9.x)	);
	
	PSIn.LightDir1.xyz = mul(PSIn.LightDir1.xyz,rot);
	
	rot[0] = float3(	cos(vecSkill9.y),		-sin(vecSkill9.y),	0.0);
	rot[1] = float3(	sin(vecSkill9.y),		cos(vecSkill9.y),		0.0);
	rot[2] = float3(	0.0,					 	0.0,						1.0);
	
	PSIn.LightDir1.xyz = mul(PSIn.LightDir1.xyz,rot);
	
	
	//rotate dir 2
	rot[0] = float3(	1.0,	0.0,						0.0					);
	rot[1] = float3(	0.0,	cos(vecSkill13.y),	-sin(vecSkill13.y));
	rot[2] = float3(	0.0,	sin(vecSkill13.y),	cos(vecSkill13.y)	);
	
	PSIn.LightDir2.xyz = mul(PSIn.LightDir2.xyz,rot);
	
	rot[0] = float3(	cos(vecSkill9.w) ,	0.0,	sin(vecSkill9.w)	);
	rot[1] = float3(	0.0,						1.0,	0.0					);
	rot[2] = float3(	-sin(vecSkill9.w),	0.0,	cos(vecSkill9.w)	);
	
	PSIn.LightDir2.xyz = mul(PSIn.LightDir2.xyz,rot);
	
	rot[0] = float3(	cos(vecSkill13.x),		-sin(vecSkill13.x),	0.0);
	rot[1] = float3(	sin(vecSkill13.x),		cos(vecSkill13.x),	0.0);
	rot[2] = float3(	0.0,					 		0.0,						1.0);
	
	PSIn.LightDir2.xyz = mul(PSIn.LightDir2.xyz,rot);
	
	
	//Light1
	float RealDistance = distance(PSIn.WorldPos.xyz,vecSkill1.xyz);
	float FadeOut = saturate(1.0-pow(RealDistance/vecSkill1.w,3));
	float3 Color1 = (1.0-texCUBE(Depth1Sampler, float4(PSIn.LightDir1.xyz, 1.0f)).rgb)*FadeOut*vecSkill17.x;
	float final_fadeout = FadeOut;
	
	//Light2
	RealDistance = distance(PSIn.WorldPos.xyz,vecSkill5.xyz);
	FadeOut = saturate(1.0-pow(RealDistance/vecSkill5.w,3));
	float3 Color2 = (1.0-texCUBE(Depth2Sampler, float4(PSIn.LightDir2.xyz, 1.0f)).rgb)*FadeOut*vecSkill17.y;
	final_fadeout += FadeOut;
	
	return 1.0-float4(saturate(Color1+Color2),final_fadeout);
}


// Technique
technique ShadowedScene
{
	pass Pass0
	{
		ZWriteEnable = True;
		AlphaBlendEnable = False;
		AlphaTestEnable = False;
		
		VertexShader = compile vs_3_0 ShadowedSceneVertexShader();
		PixelShader = compile ps_3_0 ShadowedScenePixelShader();
	}
}