//////////////////////////////////////////////////
//////////VSM Shader for 4 Pointlights////////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////24.12.2008////////////////by Nils Daumann/////
///////////////www.ssmasters.de///////////////////
//////////////////////////////////////////////////

// Constants given by the engine
float4x4 matWorldViewProj;
float4x4 matWorld;

float4 vecLightPos[8];
float4 vecSkill1;
float4 vecSkill5;
float4 vecSkill9;
float4 vecSkill13;
float4 vecSkill17;

float4 vecLight;
float Ambient = 0.0;


// Textures
Texture mtlSkin1;
Texture mtlSkin2;
Texture mtlSkin3;
Texture mtlSkin4;

samplerCUBE Depth1Sampler = sampler_state
{
	texture = <mtlSkin1>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = wrap;
	AddressV = wrap;
};

samplerCUBE Depth2Sampler = sampler_state
{
	texture = <mtlSkin2>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = wrap;
	AddressV = wrap;
};

samplerCUBE Depth3Sampler = sampler_state
{
	texture = <mtlSkin3>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = wrap;
	AddressV = wrap;
};

samplerCUBE Depth4Sampler = sampler_state
{
	texture = <mtlSkin4>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = wrap;
	AddressV = wrap;
};

texture entSkin1;
sampler ColorMapSampler = sampler_state		// Color map sampler.
{
	Texture = <entSkin1>;
	MipFilter = LINEAR;
};

// Structs
struct SSceneVertexToPixel
{
	float4 Position	: POSITION;
	float3 TexCoords	: TEXCOORD0;
	float4 WorldPos	: TEXCOORD1;
	float4 LightDir1	: TEXCOORD2;
	float4 LightDir2	: TEXCOORD3;
	float4 LightDir3	: TEXCOORD4;
	float4 LightDir4	: TEXCOORD5;
};

// Vertex Shader
SSceneVertexToPixel ShadowedSceneVertexShader( float4 inPos : POSITION, float2 inTexCoords : TEXCOORD0, float3 inNormal : NORMAL)
{
	SSceneVertexToPixel Output = (SSceneVertexToPixel)0;
	
	Output.Position = mul(inPos, matWorldViewProj);
	Output.TexCoords.xy = inTexCoords;
	
	//Fading
	Output.WorldPos  = mul(inPos, matWorld);
	
	//cube-projection
	Output.LightDir1.xyz = vecSkill1.xyz - Output.WorldPos.xyz;
	Output.LightDir2.xyz = vecSkill5.xyz - Output.WorldPos.xyz;
	Output.LightDir3.xyz = vecSkill9.xyz - Output.WorldPos.xyz;
	Output.LightDir4.xyz = vecSkill13.xyz - Output.WorldPos.xyz;
	
	return Output;
}

// Pixel Shader
float VSM_FILTER(float2 moments, float fragDepth)
{
	fragDepth -= 0.75;
	float Ex_2 = moments.x * moments.x;
	float variance = moments.y - Ex_2;
	float mD = moments.x - fragDepth;
	float mD_2 = mD * mD;
	float p = variance / (variance + mD_2);

	return 1.0-max(p, fragDepth <= moments.x );
}

float4 ShadowedScenePixelShader(SSceneVertexToPixel PSIn) : COLOR
{
	clip(tex2D(ColorMapSampler,PSIn.TexCoords.xy).a-0.5);
	
	float fadeout_final = 0;
	
	//Light1
	float RealDistance = distance(PSIn.WorldPos.xyz,vecSkill1.xyz);
	float FadeOut = saturate(1.0-pow(RealDistance/vecSkill1.w,3));
	float2 moments = texCUBE(Depth1Sampler, -float4(PSIn.LightDir1.xyz, 1.0f)).rg;
	float4 Shadow1 = VSM_FILTER(moments,RealDistance)*FadeOut*vecSkill17.x;
	fadeout_final += FadeOut;
	
	//Light2
	RealDistance = distance(PSIn.WorldPos.xyz,vecSkill5.xyz);
	FadeOut = saturate(1.0-pow(RealDistance/vecSkill5.w,3));
	moments = texCUBE(Depth2Sampler, -float4(PSIn.LightDir2.xyz, 1.0f)).rg;
	float4 Shadow2 = VSM_FILTER(moments,RealDistance)*FadeOut*vecSkill17.y;
	fadeout_final += FadeOut;
	
	//Light3
	RealDistance = distance(PSIn.WorldPos.xyz,vecSkill9.xyz);
	FadeOut = saturate(1.0-pow(RealDistance/vecSkill9.w,3));
	moments = texCUBE(Depth3Sampler, -float4(PSIn.LightDir3.xyz, 1.0f)).rg;
	float4 Shadow3 = VSM_FILTER(moments,RealDistance)*FadeOut*vecSkill17.z;
	fadeout_final += FadeOut;
	
	//Light4
	RealDistance = distance(PSIn.WorldPos.xyz,vecSkill13.xyz);
	FadeOut = saturate(1.0-pow(RealDistance/vecSkill13.w,3));
	moments = texCUBE(Depth4Sampler, -float4(PSIn.LightDir4.xyz, 1.0f)).rg;
	float4 Shadow4 = VSM_FILTER(moments,RealDistance)*FadeOut*vecSkill17.w;
	fadeout_final += FadeOut;
	
	return 1.0-float4(saturate(Shadow1+Shadow2+Shadow3+Shadow4).r,fadeout_final,0,0);
}


// Technique
technique ShadowedScene
{
	pass Pass0
	{
		ZWriteEnable = True;
		AlphaBlendEnable = False;
		AlphaTestEnable = False;
		
		VertexShader = compile vs_3_0 ShadowedSceneVertexShader();
		PixelShader = compile ps_3_0 ShadowedScenePixelShader();
	}
}