//////////////////////////////////////////////////
//////////VSM Shader for 2 Spotlights////////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////24.12.2008////////////////by Nils Daumann/////
///////////////www.ssmasters.de///////////////////
//////////////////////////////////////////////////

// Constants given by the engine
float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matEffect1;
float4x4 matEffect2;

float4 vecSkill1;
float4 vecSkill5;
float4 vecSkill9;
float4 vecSkill13;
float4 vecSkill17;


// Textures
Texture mtlSkin1;
Texture mtlSkin2;
Texture mtlSkin3;
Texture mtlSkin4;

sampler2D Proj1Sampler = sampler_state
{
	texture = <mtlSkin1>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = clamp;
	AddressV = clamp;
};

sampler2D Depth1Sampler = sampler_state
{
	texture = <mtlSkin2>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = clamp;
	AddressV = clamp;
};

sampler2D Proj2Sampler = sampler_state
{
	texture = <mtlSkin3>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = clamp;
	AddressV = clamp;
};

sampler2D Depth2Sampler = sampler_state
{
	texture = <mtlSkin4>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = clamp;
	AddressV = clamp;
};

texture entSkin1;
sampler ColorMapSampler = sampler_state		// Color map sampler.
{
	Texture = <entSkin1>;
	MipFilter = LINEAR;
};

// Structs
struct SSceneVertexToPixel
{
	float4 Position	: POSITION;
	float3 TexCoords	: TEXCOORD0;
	float4 WorldPos	: TEXCOORD1;
	float4 ProjCoord1	: TEXCOORD2;
	float4 ProjCoord2	: TEXCOORD3;
};

// Vertex Shader
SSceneVertexToPixel ShadowedSceneVertexShader( float4 inPos : POSITION, float2 inTexCoords : TEXCOORD0, float3 inNormal : NORMAL)
{
	SSceneVertexToPixel Output = (SSceneVertexToPixel)0;
	
	Output.Position = mul(inPos, matWorldViewProj);
	Output.TexCoords.xy = inTexCoords;
	
	//Fading
	Output.WorldPos  = mul(inPos, matWorld);
	
	//projection
	Output.ProjCoord1 = mul(Output.WorldPos,matEffect1);
	Output.ProjCoord2 = mul(Output.WorldPos,matEffect2);
	
	return Output;
}

// Pixel Shader
float VSM_FILTER(float2 moments, float fragDepth)
{
	fragDepth -= 0.75;
	float Ex_2 = moments.x * moments.x;
	float variance = moments.y - Ex_2;
	float mD = moments.x - fragDepth;
	float mD_2 = mD * mD;
	float p = variance / (variance + mD_2);

	return max(p, fragDepth <= moments.x );
}

float4 ShadowedScenePixelShader(SSceneVertexToPixel PSIn) : COLOR
{
	clip(tex2D(ColorMapSampler,PSIn.TexCoords.xy).a-0.5);
	
	float2 Coords1 = PSIn.ProjCoord1.xy/PSIn.ProjCoord1.w;
	float2 Coords2 = PSIn.ProjCoord2.xy/PSIn.ProjCoord2.w;
	
	float RealDistance;
	float2 moments;
	float3 Shadow1 = 1.0;
	float3 Shadow2 = 1.0;
	
	float FadeOut = 0;
	float final_FadeOut = 0;
	
	//Light1
	if(!(Coords1.x > 1.0 || Coords1.y > 1.0 || Coords1.x < 0 || Coords1.y < 0) &&
		PSIn.ProjCoord1.w >= 0)
	{
		RealDistance = distance(PSIn.WorldPos.xyz,vecSkill1.xyz);
		FadeOut = saturate(1.0-pow(RealDistance/vecSkill1.w,3));
		moments = tex2D(Depth1Sampler, Coords1).rg;
		Shadow1 = VSM_FILTER(moments,RealDistance)*vecSkill17.x;
		Shadow1 *= tex2D(Proj1Sampler, Coords1).rgb;
		Shadow1 = 1.0-Shadow1;
		Shadow1 *= FadeOut;
		final_FadeOut += FadeOut;
	}
	
	//Light2
	if(
		!(Coords2.x > 1.0 || Coords2.y > 1.0 || Coords2.x < 0 || Coords2.y < 0) &&
		PSIn.ProjCoord2.w >= 0)
	{
		RealDistance = distance(PSIn.WorldPos.xyz,vecSkill5.xyz);
		FadeOut = saturate(1.0-pow(RealDistance/vecSkill5.w,3));
		moments = tex2D(Depth2Sampler, Coords2).rg;
		Shadow2 = VSM_FILTER(moments,RealDistance)*vecSkill17.y;
		Shadow2 *= tex2D(Proj2Sampler, Coords2).rgb;
		Shadow2 = 1.0-Shadow2;
		Shadow2 *= FadeOut;
		final_FadeOut += FadeOut;
	}
	
	if(((Coords1.x > 1.0 || Coords1.y > 1.0 || Coords1.x < 0 || Coords1.y < 0) || (1.0-all(vecSkill5.w)) &&
		(Coords2.x > 1.0 || Coords2.y > 1.0 || Coords2.x < 0 || Coords2.y < 0)) || (1.0-all(vecSkill1.w)) ||
		(PSIn.ProjCoord1.w+PSIn.ProjCoord2.w) < 0)
	{
		Shadow1 = 0.0;
		Shadow2 = 0.0;
		
		final_FadeOut = 0.0;
	}
	
	return float4(2.0-(Shadow1+Shadow2+saturate(1.0-all(vecSkill1.w))+saturate(1.0-all(vecSkill5.w))),1.0-final_FadeOut);
}


// Technique
technique ShadowedScene
{
	pass Pass0
	{
		ZWriteEnable = True;
		AlphaBlendEnable = False;
		AlphaTestEnable = False;
		
		VertexShader = compile vs_3_0 ShadowedSceneVertexShader();
		PixelShader = compile ps_3_0 ShadowedScenePixelShader();
	}
}