//////////////////////////////////////////////////
////////////////Sun Shadowmapping/////////////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////24.02.2009////////////////by Nils Daumann/////
///////////////www.ssmasters.de///////////////////
//////////////////////////////////////////////////

#ifndef SM_SUN
#define SM_SUN

BMAP* SM_Sun_noise = "noise.dds";

var SM_Sun_Active_var = 0;

float SM_Sun_matrix[64];

dllfunction SM_SplitMatrix_create(VIEW *CamView, VIEW *LightView, var clipnear, var clipfar);

void SMSun_setViewProj();

MATERIAL* SM_Sun_Main_mat =
{
	effect = "SM_Sun_main.fx";
	flags = ENABLE_VIEW;
}

#ifndef SM_COMBINE
#define SM_COMBINE
	MATERIAL* SM_Combine_mat =
	{
		effect = "SM_Combine.fx";
	}
#endif

void SM_Sun_delete()
{
	SM_Sun_Active_var = 0;
}

void SMSun_getViewProj()
{
	mat_effect1 = &SM_Sun_matrix[0];
	mat_effect2 = &SM_Sun_matrix[16];
	mat_effect3 = &SM_Sun_matrix[32];
	mat_effect4 = &SM_Sun_matrix[48];
	
	VIEW* TempView = (VIEW*)mtl.skill1;
	SM_SplitMatrix_create(TempView,render_view,mtl.skill2,mtl.skill3);
}

void SMSun_setViewProj()
{
	if(mtl.skin1 == 0){return;}
	
	float fTexAdj[16] =
	{
		0.5, 0.0, 0.0, 0.0,
		0.0,-0.5, 0.0, 0.0,
		0.0, 0.0, 1.0, 0.0,
		0.0, 0.0, 0.0, 1.0
	};
	
	fTexAdj[12] = 0.5 + (0.5/floatv(mtl.skin1.width));
	fTexAdj[13] = 0.5 + (0.5/floatv(mtl.skin1.height));
	
	MATERIAL** TempMat = (MATERIAL**)mtl.skill4;
	
	// passes the viewprojectionmatrix calculated in the depthmaterialevent to the shader
	var i;
	for(i = 0; i < mtl.skill3; i++)
	{
		mat_set(SM_Sun_matrix[i*16],TempMat[i].matrix);
		mat_multiply(SM_Sun_matrix[i*16],fTexAdj);
	}
	
	mat_effect1 = &SM_Sun_matrix[0];
	mat_effect2 = &SM_Sun_matrix[16];
	mat_effect3 = &SM_Sun_matrix[32];
	mat_effect4 = &SM_Sun_matrix[48];
}


void SM_CreateSun(var blur, var bias, var lerpval, var fadeoutdist, var res, var bits)
{
	SM_Sun_delete();
	wait(1);
	
	var splits = 3;
	if(splits == 0)
		return;
	
	splits = clamp(splits,1,4);
	
	proc_mode = PROC_LATE;
	
	vec_set(sky_color,vector(255,255,255));
   vec_set(screen_color,vector(255,255,255));
	
	VIEW** shadowviews;
	shadowviews = malloc(sizeof(VIEW*)*splits);
	
	MATERIAL** shadowdepthmats;
	shadowdepthmats = malloc(sizeof(MATERIAL*)*splits);
	
	VIEW* mainview;
	mainview = view_create(-1);
	
	SM_Sun_Main_mat.skill3 = splits;
	SM_Sun_Main_mat.skill4 = (var)shadowdepthmats;
	SM_Sun_Main_mat.skill5 = floatv(fadeoutdist);
	SM_Sun_Main_mat.skill6 = floatv(1/res);
	SM_Sun_Main_mat.skill7 = floatv(blur);
	
	SM_Sun_Main_mat.skill13 = floatv(bias);		//bias depth 1
	SM_Sun_Main_mat.skill14 = floatv(bias*2.0);	//bias depth 2
	SM_Sun_Main_mat.skill15 = floatv(bias*4.0);	//bias depth 3
	
	var shadowdist;
	shadowdist = 2.7*fadeoutdist/(pow(splits+1,4)*(lerpval)+(splits+1)*(1.0-lerpval));
	
	var i;
	var poss_lin;
	var poss_exp;
	for(i = 0; i < splits; i++)
	{
		shadowdepthmats[i] = mtl_create();
		effect_load(shadowdepthmats[i],"SM_Sun_depth.fx");
		set(shadowdepthmats[i],ENABLE_VIEW);
		shadowdepthmats[i].event = SMSun_getViewProj;
		shadowdepthmats[i].skill1 = (var)camera;
		
		poss_lin = camera.clip_near+shadowdist*i;
		poss_exp = camera.clip_near+shadowdist*i*i*i*i;
		shadowdepthmats[i].skill2 = poss_lin*(1.0-lerpval)+poss_exp*(lerpval);
		
		poss_lin = camera.clip_near+shadowdist*(i+1);
		poss_exp = camera.clip_near+shadowdist*(i+1)*(i+1)*(i+1)*(i+1);
		shadowdepthmats[i].skill3 = poss_lin*(1.0-lerpval)+poss_exp*(lerpval);
		
		shadowviews[i] = view_create(-2);
		set(shadowviews[i],ISOMETRIC);
		shadowviews[i].arc = 30+60*i;
		shadowviews[i].material = shadowdepthmats[i];
		shadowviews[i].size_x = res;
		shadowviews[i].size_y = res;
		shadowviews[i].clip_near = 0;
		shadowviews[i].clip_far = 200000;
		set(shadowviews[i],VISIBLE|NOFLAG1);
		
		switch(i)
		{
			case 0:
				SM_Sun_Main_mat.skin1 = bmap_createblack(res,res,bits);
				shadowviews[i].bmap = SM_Sun_Main_mat.skin1;
				break;
				
			case 1:
				SM_Sun_Main_mat.skin2 = bmap_createblack(res,res,bits);
				shadowviews[i].bmap = SM_Sun_Main_mat.skin2;
				break;
				
			case 2:
				SM_Sun_Main_mat.skin3 = bmap_createblack(res,res,bits);
				shadowviews[i].bmap = SM_Sun_Main_mat.skin3;
				break;
				
			case 3:
				SM_Sun_Main_mat.skin4 = bmap_createblack(res,res,bits);
				shadowviews[i].bmap = SM_Sun_Main_mat.skin4;
				break;
				
			default:
				return;
		}
	}
	
	mainview.material = SM_Sun_Main_mat;
	mainview.size_x = screen_size.x;
	mainview.size_y = screen_size.y;
	SM_Combine_mat.skin2 = bmap_createblack(screen_size.x,screen_size.y,24);
	mainview.bmap = SM_Combine_mat.skin2;
	set(mainview,VISIBLE);
	
	PP_Remove(SM_Combine_mat,camera,NULL);
	PP_Add(SM_Combine_mat,camera,NULL);
	
	SM_Combine_mat.skill2 = floatv(1);
	SM_Sun_Main_mat.event = SMSun_setViewProj;
	
	SM_Sun_Active_var = 1;
	
	VECTOR temp;
	VECTOR temp2;
	while(SM_Sun_Active_var)
	{
		vec_set(mainview.x,camera.x);
		vec_set(mainview.pan,camera.pan);
		mainview.genius = camera.genius;
		mainview.clip_near = camera.clip_near;
		mainview.clip_far = camera.clip_far;
		mainview.arc = camera.arc;
		
		vec_set(shadowviews[0].x,sun_pos.x);
		
		for(i = 0; i < splits; i++)
		{
			vec_set(temp,camera.x);
			vec_set(temp2,vector((shadowdepthmats[i].skill2+shadowdepthmats[i].skill2)*0.5,0,0));
			vec_rotate(temp2,camera.pan);
			vec_add(temp,temp2);
			vec_sub(temp,shadowviews[i].x);
			vec_to_angle(shadowviews[i].pan,temp);
		}
		
		for(i = 1; i < splits; i++)
		{
			vec_set(shadowviews[i].x,shadowviews[0].x);
		}
		
		SM_Sun_Main_mat.skill9 = floatv(sun_pos.x);
		SM_Sun_Main_mat.skill10 = floatv(sun_pos.z);
		SM_Sun_Main_mat.skill11 = floatv(sun_pos.y);
		
		wait(1);
	}
	
	SM_Combine_mat.skill2 = floatv(0);
	SM_Sun_Main_mat.event = NULL;
	
	for(i = splits-1; i >= 0; i--)
	{
		ptr_remove(shadowviews[i]);
		free(&(shadowviews[i]));
	}
	
	for(i = splits-1; i >= 0; i--)
	{
		ptr_remove(shadowdepthmats[i]);
		free(&(shadowdepthmats[i]));
	}
}

#endif
