//////////////////////////////////////////////////
////////Simple Depthmap Shader for the Sun////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////24.12.2008////////////////by Nils Daumann/////
///////////////www.ssmasters.de///////////////////
//////////////////////////////////////////////////

// Constants given by the engine
float4x4 matWorld;
float4x4 matMtl;
float4x4 matView;
float4x4 matProj;
float4x4 matWorldViewProj;
float4x4 matViewProj;
float4x4 matEffect4;

float4 vecViewPos;
float4 vecSunPos;

// Textures
texture entSkin1;
sampler ColorMapSampler = sampler_state
{
	Texture = <entSkin1>;
	MipFilter = LINEAR;
};

// Structs
struct SMapVertexToPixel
{
    float4 Position     : POSITION;
    float2 TexCoords    : TEXCOORD0;
    float4 Pos				: TEXCOORD1;
};

// Vertex Shader
SMapVertexToPixel ShadowMapVertexShader(float4 inPos : POSITION, float2 texcoords : TEXCOORD0)
{
   SMapVertexToPixel Output;
	
	Output.Pos = mul(inPos,matWorld);
   Output.Position = mul(Output.Pos,matMtl);//mul(matView,matProj)));
//	float4 texelsize = mul(matEffect4,(Output.Position*0.5+0.5+float4(1/1280,1/1024,0,0)*Output.Position.w)*2-1);

//	texelsize.x = length(texelsize.xyz-Output.Pos.xyz);
//	texelsize.x *= texelsize.x;
	
//	float a = 0.0000001;
//	float b = 0.0000001;

//	Output.Pos = Output.Pos + vecSunPos*(a + b*texelsize.x),

/*P orig is the original point position,
L is the light vector direction in world space,
Ltexel is the texel size in world space,
M is the final shadow map matrix, and
a and b are bias coefficients.*/

/*The texel size in world space could be approximately computed with simple matrix calculations.
First, transform the point into shadow map space,
and then shift this point by the texel size without changing depth.
Next, transform it back into world space and square the length of
the difference between this point and the original one.
This gives us Ltexel  :
and S x and S y are shadow map resolutions.*/

   Output.TexCoords.xy = texcoords;

   return Output;
}

// Pixel Shader
float4 ShadowMapPixelShader(SMapVertexToPixel PSIn) : COLOR0
{
	clip(tex2D(ColorMapSampler,PSIn.TexCoords).a-0.2);
	
	float depth = length(vecViewPos.xyz-PSIn.Pos.xyz);

	float4 Color = 0.0;
	Color.r = depth;
	Color.g = depth*depth;

	return Color;//tex2D(ColorMapSampler,PSIn.TexCoords);//Color;
}


// Technique
technique ShadowMap
{
	pass Pass0
	{
		CullMode = CCW;
		ZWriteEnable = True;
		AlphaBlendEnable = False;
		
		VertexShader = compile vs_2_0 ShadowMapVertexShader();
		PixelShader = compile ps_2_0 ShadowMapPixelShader();
	}
}