//////////////////////////////////////////////////
////////////////Sun Shadowshader//////////////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////24.12.2008////////////////by Nils Daumann/////
///////////////www.ssmasters.de///////////////////
//////////////////////////////////////////////////

//Variables
float4x4 matWorldViewProj;	
float4x4 matWorld;

float4 vecSkill1;
float4 vecSkill5;
float4 vecSkill9;
float4 vecSkill13;

float4x4 matEffect1;
float4x4 matEffect2;
float4x4 matEffect3;

float4 vecSunPos;
float4 vecViewPos;


//Textures
texture entSkin1; //this is the color map

texture mtlSkin1;
texture mtlSkin2;
texture mtlSkin3;
texture mtlSkin4;

texture SM_Sun_noise_bmap;


//Sampler
sampler colorMap = sampler_state
{
   Texture = <entSkin1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = wrap;
   AddressV  = wrap;
};

sampler Depth1Sampler = sampler_state
{
	texture = <mtlSkin1>;
	magfilter = Point;
	minfilter = Point;
	mipfilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};

sampler Depth2Sampler = sampler_state
{
	texture = <mtlSkin2>;
	magfilter = Point;
	minfilter = Point;
	mipfilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};

sampler Depth3Sampler = sampler_state
{
	texture = <mtlSkin3>;
	magfilter = Point;
	minfilter = Point;
	mipfilter = None;
	AddressU = Border;
	AddressV = Border;
	BorderColor = 0xffffff;
};

sampler NoiseSampler = sampler_state
{
	texture = <SM_Sun_noise_bmap>;
	magfilter = None;
	minfilter = None;
	mipfilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
};


//Structs
struct VS_TO_PS
{
	float4 Pos : POSITION;
	float3 Tex : TEXCOORD0;
	float4 PosWorld : TEXCOORD1;
	
	float4 ShadowMapSamplingPos1	: TEXCOORD3;
	float4 ShadowMapSamplingPos2	: TEXCOORD4;
	float4 ShadowMapSamplingPos3	: TEXCOORD5;
};


//Vertexshader
VS_TO_PS VS(float4 inPos		: POSITION,
				float2 inTex		: TEXCOORD0)
{
	VS_TO_PS Out = (VS_TO_PS)0;
	
	//Calculate the final vertexposition
	Out.Pos = mul(inPos, matWorldViewProj);
	
	//Get the vertexposition in worldspace
	Out.PosWorld = mul(inPos, matWorld);
	
	//Pass the TexCoords
	Out.Tex.xy = inTex;
	
	Out.Tex.z = saturate(distance(vecViewPos,Out.PosWorld)/vecSkill5.x);
	Out.Tex.z = pow(Out.Tex.z,10);
	Out.Tex.z = 1.0-Out.Tex.z;
	
	Out.ShadowMapSamplingPos1 = mul(Out.PosWorld,matEffect1);
	Out.ShadowMapSamplingPos1.z = 1.0/vecSkill13.x;/*1.0f/distance(Out.PosWorld.xyz,
														mul(	matEffect1,Out.ShadowMapSamplingPos1+
																float4(vecSkill5.y,0,0,0)*Out.ShadowMapSamplingPos1.w).xyz);*/
	
	Out.ShadowMapSamplingPos2 = mul(Out.PosWorld,matEffect2);
	Out.ShadowMapSamplingPos2.z = 1.0/vecSkill13.y;/*0.5f/distance(Out.PosWorld.xyz,
														mul(	matEffect2,Out.ShadowMapSamplingPos2+
																float4(vecSkill5.y,0,0,0)*Out.ShadowMapSamplingPos2.w).xyz);*/
	
	Out.ShadowMapSamplingPos3 = mul(Out.PosWorld,matEffect3);
	Out.ShadowMapSamplingPos3.z = 1.0/vecSkill13.z;/*0.25f/distance(Out.PosWorld.xyz,
														mul(	matEffect3,Out.ShadowMapSamplingPos3+
																float4(vecSkill5.y,0,0,0)*Out.ShadowMapSamplingPos3.w).xyz);*/
	
	return Out;
}

//Pixelshader
float4 PS(VS_TO_PS In) : COLOR0
{
	float4 Color = tex2D(colorMap,In.Tex.xy);
	clip(Color.a-0.3);
	
	// Pack all shadow results into a single float4...
	float2 shadowProjVec[3];
	shadowProjVec[0].xy = In.ShadowMapSamplingPos1.xy/In.ShadowMapSamplingPos1.w;
	shadowProjVec[1].xy = In.ShadowMapSamplingPos2.xy/In.ShadowMapSamplingPos2.w;
	shadowProjVec[2].xy = In.ShadowMapSamplingPos3.xy/In.ShadowMapSamplingPos3.w;
	
	float4 offset[3];
	offset[0].xy = float2(1,0)*1/512;
	offset[0].zw = float2(0,1)*1/512;
	offset[1].xy = float2(1,1)*1/512;
	offset[1].zw = float2(-2,0)*1/512;
	offset[2].xy = float2(0,-2)*1/512;
	offset[2].zw = float2(-2,-2)*1/512;
	
	offset[0].xyzw = tex2D(NoiseSampler,In.PosWorld.xy+In.PosWorld.z+offset[0].xy)*vecSkill5.y*vecSkill5.z;
	offset[1].xyzw = tex2D(NoiseSampler,In.PosWorld.xy+In.PosWorld.z+offset[1].xy)*vecSkill5.y*vecSkill5.z;
	offset[2].xyzw = tex2D(NoiseSampler,In.PosWorld.xy+In.PosWorld.z+offset[2].xy)*vecSkill5.y*vecSkill5.z;
	
	float i;
	float4 finalDepth[2];
	float4 fShadow[2];
	
	float realdepth = distance(vecSkill9.xyz,In.PosWorld.xyz);
	
	if(shadowProjVec[0].x > 0.0 && shadowProjVec[0].x < 1.0 && shadowProjVec[0].y > 0.0 && shadowProjVec[0].y < 1.0)
	{
		finalDepth[0].r = tex2D(Depth1Sampler, shadowProjVec[0].xy+offset[0].xy*In.ShadowMapSamplingPos1.z).r;
		finalDepth[0].g = tex2D(Depth1Sampler, shadowProjVec[0].xy+offset[0].zw*In.ShadowMapSamplingPos1.z).r;
		finalDepth[0].b = tex2D(Depth1Sampler, shadowProjVec[0].xy+offset[1].xy*In.ShadowMapSamplingPos1.z).r;
		finalDepth[0].a = tex2D(Depth1Sampler, shadowProjVec[0].xy+offset[1].zw*In.ShadowMapSamplingPos1.z).r;
		
		finalDepth[1].r = tex2D(Depth1Sampler, shadowProjVec[0].xy+offset[2].xy*In.ShadowMapSamplingPos1.z).r;
		finalDepth[1].g = tex2D(Depth1Sampler, shadowProjVec[0].xy+offset[2].zw*In.ShadowMapSamplingPos1.z).r;
		
		realdepth -= vecSkill13.x;
		i = 0;
	}else
	{
		if(shadowProjVec[1].x > 0.0 && shadowProjVec[1].x < 1.0 && shadowProjVec[1].y > 0.0 && shadowProjVec[1].y < 1.0)
		{
			finalDepth[0].r = tex2D(Depth2Sampler, shadowProjVec[1].xy+offset[0].xy*In.ShadowMapSamplingPos2.z).r;
			finalDepth[0].g = tex2D(Depth2Sampler, shadowProjVec[1].xy+offset[0].zw*In.ShadowMapSamplingPos2.z).r;
			finalDepth[0].b = tex2D(Depth2Sampler, shadowProjVec[1].xy+offset[1].xy*In.ShadowMapSamplingPos2.z).r;
			finalDepth[0].a = tex2D(Depth2Sampler, shadowProjVec[1].xy+offset[1].zw*In.ShadowMapSamplingPos2.z).r;
			
			finalDepth[1].r = tex2D(Depth2Sampler, shadowProjVec[1].xy+offset[2].xy*In.ShadowMapSamplingPos2.z).r;
			finalDepth[1].g = tex2D(Depth2Sampler, shadowProjVec[1].xy+offset[2].zw*In.ShadowMapSamplingPos2.z).r;
			
			realdepth -= vecSkill13.y;
			i = 1;
		}else
		{
			finalDepth[0].r = tex2D(Depth3Sampler, shadowProjVec[2].xy+offset[0].xy*In.ShadowMapSamplingPos3.z).r;
			finalDepth[0].g = tex2D(Depth3Sampler, shadowProjVec[2].xy+offset[0].zw*In.ShadowMapSamplingPos3.z).r;
			finalDepth[0].b = tex2D(Depth3Sampler, shadowProjVec[2].xy+offset[1].xy*In.ShadowMapSamplingPos3.z).r;
			finalDepth[0].a = tex2D(Depth3Sampler, shadowProjVec[2].xy+offset[1].zw*In.ShadowMapSamplingPos3.z).r;
			
			finalDepth[1].r = tex2D(Depth3Sampler, shadowProjVec[2].xy+offset[2].xy*In.ShadowMapSamplingPos3.z).r;
			finalDepth[1].g = tex2D(Depth3Sampler, shadowProjVec[2].xy+offset[2].zw*In.ShadowMapSamplingPos3.z).r;
			
			realdepth -= vecSkill13.z;
			i = 2;
		}
	}
	
	fShadow[0] = (finalDepth[0] > realdepth);
	fShadow[1].rg = (finalDepth[1].rg > realdepth);
	
	float fLightingFactor = dot(fShadow[0],0.166666667)+dot(fShadow[1].rg,0.166666667);
	
	return 1.0-(1.0-fLightingFactor)*In.Tex.z;
}


//Technique
technique NormalMapping
{
	pass P0
	{
		alphablendenable = false;
		alphatestenable = false;
		zWriteEnable = true;
		
		// compile shaders
		VertexShader = compile vs_3_0 VS();
		PixelShader  = compile ps_3_0 PS();
	}
}

/*
// Calculates the shadow occlusion using bilinear PCF
float CalcShadowTermPCF(float fLightDepth, float2 vTexCoord)
{
	float fShadowTerm = 0.0f;

	// transform to texel space
	float2 vShadowMapCoord = g_vShadowMapSize * vTexCoord;
    
	// Determine the lerp amounts           
	float2 vLerps = frac(vShadowMapCoord);

	// read in bilerp stamp, doing the shadow checks
	float fSamples[4];
	
	fSamples[0] = (tex2D(shadowMapSampler, vTexCoord).x + BIAS < fLightDepth) ? 0.0f: 1.0f;  
	fSamples[1] = (tex2D(shadowMapSampler, vTexCoord + float2(1.0/g_vShadowMapSize.x, 0)).x + BIAS < fLightDepth) ? 0.0f: 1.0f;  
	fSamples[2] = (tex2D(shadowMapSampler, vTexCoord + float2(0, 1.0/g_vShadowMapSize.y)).x + BIAS < fLightDepth) ? 0.0f: 1.0f;  
	fSamples[3] = (tex2D(shadowMapSampler, vTexCoord + float2(1.0/g_vShadowMapSize.x, 1.0/g_vShadowMapSize.y)).x + BIAS < fLightDepth) ? 0.0f: 1.0f;  
    
	// lerp between the shadow values to calculate our light amount
	fShadowTerm = lerp( lerp( fSamples[0], fSamples[1], vLerps.x ),
							  lerp( fSamples[2], fSamples[3], vLerps.x ),
							  vLerps.y );							  
								
	return fShadowTerm;								 
}


float CalcShadowTermSoftPCF(float fLightDepth, float2 vTexCoord, int iSqrtSamples)
{
	float fShadowTerm = 0.0f;  
		
	float fRadius = (iSqrtSamples - 1.0f) / 2;
	float fWeightAccum = 0.0f;
	
	for (float y = -fRadius; y <= fRadius; y++)
	{
		for (float x = -fRadius; x <= fRadius; x++)
		{
			float2 vOffset = 0;
			vOffset = float2(x, y);				
			vOffset /= g_vShadowMapSize;
			float2 vSamplePoint = vTexCoord + vOffset;			
			float fDepth = tex2D(shadowMapSampler, vSamplePoint).x;
			float fSample = (fLightDepth <= fDepth + BIAS);
			
			// Edge tap smoothing
			float xWeight = 1;
			float yWeight = 1;
			
			if (x == -fRadius)
				xWeight = 1 - frac(vTexCoord.x * g_vShadowMapSize.x);
			else if (x == fRadius)
				xWeight = frac(vTexCoord.x * g_vShadowMapSize.x);
				
			if (y == -fRadius)
				yWeight = 1 - frac(vTexCoord.y * g_vShadowMapSize.y);
			else if (y == fRadius)
				yWeight = frac(vTexCoord.y * g_vShadowMapSize.y);
				
			fShadowTerm += fSample * xWeight * yWeight;
			fWeightAccum = xWeight * yWeight;
		}											
	}		
	
	fShadowTerm /= (iSqrtSamples * iSqrtSamples);
	
	return fShadowTerm;
}
*/
